# -*- coding: utf-8 -*-

import json
import os
import load_template


#__PBPrefixName__ = "GetGlobalState_pb2."

__TemplateCodeFileName__ = "template_code.txt"
__TemplateFileHeadName__ = "template_head.txt"
__TemplateRstName__ = "rst_template.txt"

template = load_template.FutuTemplate("function.template")



def change_variable_name(listx):  # 修改变量名
    """把骆驼变量名变成下划线的方式（全大写）"""
    listy = listx[0]
    for i in range(1, len(listx) - 1):

        if listx[i] == '_' or listx[i - 1] == '_' or listx[i + 1] == '_':
            listy += listx[i]

        elif listx[i].isupper() and not listx[i - 1].isupper():  # 加'_',当前为大写，前一个字母为小写
            listy += '_'
            listy += listx[i]

        elif listx[i].isupper() and listx[i - 1].isupper() and listx[i + 1].islower():
            # 加'_',当前为大写，前一个字母为小写
            listy += '_'
            listy += listx[i]
        else:
            listy += listx[i]
    if len(listx) > 1:
        listy += listx[-1]
    return listy.upper()


def code_add_space(code, space_count=1, space_str=" "):  # 逐行加空格
    """传入字符串，逐行加入空格，符合python规范"""
    ls = code.split('\n')
    ret_code = space_line = ""
    for i in range(space_count):
        space_line += space_str

    for s in ls:
        if len(s) > 0:
            ret_code += (space_line + s + "\n")
        else:
            ret_code += "\n"

    return ret_code[:-1]

class ClassItemStruct(object):

    def __init__(self, json_obj, class_type, pb_prefix_name):
        self.name = json_obj["longName"]
        self.full_name = json_obj["fullName"]
        self.description = json_obj["description"]
        self.json_obj = json_obj
        self.pb_prefix_name = pb_prefix_name
        self.class_type = class_type
        self.repeated_count = 0
        self.var_list = list()
        self.list_list = list()
        self.values = list()

        if class_type == "enums":
            for v in json_obj["values"]:
                s = EnumsItemStruct(v, self.name, self.pb_prefix_name)
                self.values.append(s)

        if class_type == "class":
            for v in json_obj["fields"]:
                s = ParameterItem(v, self)
                if s.label == "repeated":
                    #self.has_repeated = True
                    self.repeated_count += 1
                self.values.append(s)

    def get_only_repeated_item(self):
        for item in self.values:
            if item.trim_type == "list" or item.trim_type == "code_list":
                return item

    # def is_repeated_type(self, full_name):
    #     for item in self.values:
    #         if item.full_type == full_name:
    #             return True
    #     return False

    def set_parameter_class(self, class_obj):
        for item in self.values:
            item.set_class(class_obj)

    def get_function_return(self):
        var_names = list()
        for item in self.var_list:
            var_names.append(item.trim_name)
        """函数返回，主要是组成pd和解析"""
        if self.repeated_count == 1:
            str_code = '\'' + '\',\n\''.join(var_names) + '\''
            return template["list_return"].format(var_name=code_add_space(str_code, 4))
        elif self.repeated_count == 0:
            str_code = '\'' + '\',\n\''.join(var_names) + '\''
            return template["dict_return"].format(var_name=code_add_space(str_code, 4))
        elif self.repeated_count > 1:
            str_code = ""
            for item in self.list_list:
                var_names.clear()
                col_list = item.col_list
                for v in col_list:
                    var_names.append(v.trim_name)
                var_name = '\'' + '\',\n\''.join(var_names) + '\''
                trim_name = item.trim_name
                str_code += template["list_list_item_return"].format(var_name=code_add_space(var_name, 4),
                                                                     trim_name=trim_name,
                                                                     name=trim_name,
                                                                     description=item.description)

            return template["list_list_return"].format(code=str_code)


    def get_list_head(self):
        """获取列表的头部变量名代码"""
        str_code = ""
        for item in self.values:
            str_code += item.get_unpack_var()
        return str_code

    def get_unpack_code(self):
        self.var_list.clear()

        if self.repeated_count == 1:
            """如果有多个重复数据（列表项），这种方式就不适合了"""
            unpack_add_code = ""
            list_name = "ret_list"
            code, self.var_list = self.get_only_repeated_item().sub_list(list_name)
            head_code = code_add_space("{} = list()\n".format(list_name), 1)
            head_code += self.get_list_head()

            unpack_add_code += code
            head_code = code_add_space(head_code, 1)
            return head_code + unpack_add_code

        elif self.repeated_count == 0:
            unpack_code = ""
            """开始遍历赋值"""
            for item in self.values:
                unpack_code += item.get_unpack_dict_code("ret")
                self.var_list.append(item)
            return template["class_unpack_var_add"].format(unpack_code=unpack_code.rstrip('\n'))

        elif self.repeated_count > 1:
            self.list_list.clear()
            code_str = ""
            head_code = "ret_dic = dict()\n" + self.get_list_head()
            for item in self.values:
                if item.trim_type == "list":
                    self.list_list.append(item)
                    list_name = "ret_list_" + item.trim_name
                    item.list_name = list_name
                    head_code += template["class_more_list_add"].format(description=item.description,
                                                                        list_name=list_name,
                                                                        var_name=item.trim_name)
            for item in self.list_list:
                s, l = item.sub_list(item.list_name)
                item.col_list = l
                code_str += s

            return code_add_space(head_code, 1) + code_str

    def get_unpack_return(self):
        if self.repeated_count == 1:
            return "return RET_OK, \"\", ret_list"
        elif self.repeated_count == 0:
            return "return RET_OK, \"\", ret"
        elif self.repeated_count > 1:
            return "return RET_OK, \"\", ret_dic"


class BaseClass(ClassItemStruct):
    def __init__(self, full_type):
        self.name = full_type
        self.description = "base type " + full_type
        self.json_obj = None
        self.pb_prefix_name = None
        self.class_type = "class"
        self.repeated_count = 0
        self.var_list = list()
        self.vars_dict = dict()
        self.full_name = full_type
        self.values = list()


class EnumsItemStruct(object):

    def __init__(self, obj, class_name, pb_prefix_name):
        self.obj = obj
        self.class_name = class_name  # 所属的类名
        self.pb_prefix_name = pb_prefix_name
        self.full_name = obj["name"] # 原始变量名
        self.number = int(obj["number"])
        self.description = obj["description"].replace('\n', '')

        self.trim_name = self.full_name  # 整理后的变量名，主要是去掉前面的类名头
        self.pb_value = self.pb_prefix_name + self.full_name # 对应的pb里面的变量名

        if len(self.class_name) > 0 and len(self.trim_name) > 0:
            self.trim_name = self.trim_name.replace(self.class_name + "_", "")

        if self.trim_name[0] in list(map(lambda x: str(x), list(range(10)))):
            self.trim_name = self.class_name[:2] + "_" + self.trim_name

        if not isinstance(self.description, str):
            self.description = ""
        if len(self.description) == 0 and self.number == 0:
            self.description = "未知"

        self.underscore_name = change_variable_name(self.trim_name) # 整理后的变量名，把骆驼命名改成大小写命名
        if self.underscore_name in ["UNKNOW", "UNKONW", "UNKNOWN"]:
            self.underscore_name = "NONE"

        self.str = self.underscore_name # 整理后的变量值，做一些约定修整
        if self.number == 0 and self.underscore_name == "NONE":
            self.str = "N/A"


class ParameterItem(object):
    def __init__(self, obj, class_owner):
        self.obj = obj
        self.class_owner = class_owner
        self.pb_prefix_name = class_owner.pb_prefix_name
        self.self_class = None
        # self.repeated_class = None

        self.name = obj["name"] # 原始变量名
        self.full_type = obj["fullType"] # 原始变量名
        self.trim_type = obj["longType"]  # 原始变量名
        self.python_type = obj["longType"]  # python对应的变量名
        self.trim_name = change_variable_name(self.name).lower() # 整理后的名字
        self.description = obj["description"].replace('\n', '')
        self.label = obj["label"]

        self.trim()

    def trim(self):
        if self.obj is None:
            return
        # 嵌套了复杂类型list
        if self.label == "repeated":
            if self.full_type == "Qot_Common.Security":
                self.trim_type = "code_list"
                if self.trim_name == "security_list":
                    self.trim_name = "code_list"
            else:
                self.trim_type = "list"

        # 股票代码类型需要特殊处理
        elif self.full_type == "Qot_Common.Security":
            self.trim_type = "code"
            if self.trim_name == "security":
                self.trim_name = "stock_code"
        # timestamp类型需要跳过
        elif self.full_type == "double" and self.name.lower().find("timestamp") != -1:
            self.trim_type = "timestamp"

        # 时间特殊处理，因为time、code太笼统
        if self.trim_name == "time" or self.trim_name == "code":
            self.trim_name = "{}_{}".format(change_variable_name(self.class_owner.name).lower(), self.trim_name)

        # 注释太短了的不知所云
        if len(self.description) < 5:
            if len(self.class_owner.description) > 0:
                self.description = self.class_owner.description + "  " + self.description

        #名字太短了容易撞车
        if len(self.trim_name) < 5:
            owner_name = change_variable_name(self.class_owner.name).lower()
            if owner_name != "s2c" and owner_name != "c2s" and owner_name != "":
                self.trim_name = "{}_{}".format(owner_name, self.trim_name)

        if self.trim_type in ["int32", "int64", "double",
                              "float", "string", "bool", "bytes", "code_list",
                              "code", "timestamp"]:
            self.self_class = BaseClass(self.trim_type)

        elif self.full_type in ["int32", "int64", "double", "float", "string", "bool", "bytes"]:
            self.self_class = BaseClass(self.full_type)

        self.python_type = self.trim_type
        if self.trim_type in ["int32", "int64"]:
            self.python_type = "int"
        if self.trim_type in ["double", "float"]:
            self.python_type = "float"


    def get_args(self):
        return "            \"{}\": {},".format(self.trim_name, self.trim_name)

    def get_warning_filter(self):
        warning_filter = ""
        if self.trim_type == "code_list":
            warning_filter = template.get("code_list_warning_filter", 2)
        elif self.trim_type == "code":
            warning_filter = template.get("code_warning_filter", 2).format(trim_name=self.trim_name)
        return warning_filter

    def get_parameter_name(self):
        if self.trim_type == "code_list":
            return "code_list"
        elif self.trim_type == "code":
            return "stock_code"
        return self.trim_name

    def get_pack_req_filter(self):
        if self.trim_type == "code_list":
            return template.get("pack_code_list_filter", 2)
        elif self.trim_type == "code":
            return template.get("pack_code_filter", 2)
        return ""

    def get_pack_req_add(self):
        if self.trim_type == "code_list":
            return template.get("pack_code_list_add", 2).format(name=self.name)
        elif self.trim_type == "code":
            return "        req.c2s.{0}.market = market_code\n" \
                   "        req.c2s.{0}.code = stock_code".format(self.name)
        else:
            return "        req.c2s.{0} = {1}\n".format(self.name, self.trim_name)

    def set_class(self, class_obj):
        if (self.self_class is None) and (self.full_type == class_obj.full_name):
            self.self_class = class_obj
        """遍历问问子节点"""
        if (self.self_class is not None) and (not isinstance(self.self_class, BaseClass)):
            for item in self.self_class.values:
                item.set_class(class_obj)

    def get_unpack_var(self, parents_name="rsp_pb.s2c"):
        if self.trim_type == "timestamp":
            return ""
        else:
            if self.full_type == "Qot_Common.Security":
                s_code = template["class_unpack_code"].format(trim_name=self.trim_name,
                                                              name=self.name,
                                                              parents_name=parents_name)
            else:
                s_code = "#  {description} type = {trim_type}\n" \
                         "{trim_name} = {parents_name}.{name}\n".\
                    format(description=self.description,
                           trim_type=self.full_type,
                           trim_name=self.trim_name,
                           parents_name=parents_name,
                           name=self.name)
            return s_code

    def sub_list(self, list_name, sub_item_name="item", add_code=""):
        if self.self_class is None or self.self_class.repeated_count > 1:
            raise Exception("不能支持子节点有多个列表的情况")

        str_code = for_code = ""
        ret_list = list()

        if sub_item_name == "item":  # 顶级节点,把父节点的，也就是顶层父节点的数据加进去
            if self.name == "test":
                sub_item_name = "item"
            for item in self.class_owner.values:
                if item.trim_type != "list" and self.trim_type != "timestamp":
                    ret_list.append(item)
                    add_code += code_add_space(("data[\"{0}\"] = {0}\n".format(item.trim_name)), 1)

        if self.self_class.repeated_count == 1:  # 下面子节点仍然有list
            for item in self.self_class.values:
                if self.trim_type == "timestamp":
                    continue
                """加入自己那层的数据"""
                str_code += code_add_space(item.get_unpack_var(sub_item_name), 1)
                if item.trim_type != "list":
                    ret_list.append(item)
                    add_code += code_add_space(("data[\"{0}\"] = {0}\n".format(item.trim_name)), 1)
            sub_sub_item_name = "sub_" + sub_item_name

            s, l = self.self_class.get_only_repeated_item().sub_list(list_name, sub_sub_item_name, add_code)
            str_code += s
            ret_list.extend(l)

        elif self.self_class.repeated_count == 0:
            """理解为最后一层"""
            for item in self.self_class.values:
                if self.trim_type == "timestamp":
                    continue
                str_code += code_add_space(item.get_item_unpack_add(sub_item_name), 1)
                ret_list.append(item)
            str_code += add_code

        for_code = "for {sub_item} in {trim_name}:\n". \
            format(trim_name=self.trim_name,
                   sub_item=sub_item_name)
        if self.self_class.repeated_count == 0:
            for_code += "	{list_name}.append(data)\n". \
                format(list_name=list_name)

        return code_add_space(for_code + str_code, 1), ret_list



    def get_item_unpack_add(self, item_name=""):
        if self.trim_type == "timestamp" or self.trim_type == "list":
            return ""
        else:
            if len(item_name) > 0:
                item_name += "."
            if self.trim_type == "code" and item_name != "":
                s_code = "merge_qot_mkt_stock_str({parents_name}.{name}.market,{parents_name}.{name}.code)".\
                    format(name=self.name,
                           parents_name=item_name).rstrip('\n')

                return "#  {description} type = {trim_type}\n" \
                       "data[\"{trim_name}\"] = {code}\n".\
                    format(description=self.description,
                           trim_type=self.trim_type,
                           trim_name=self.trim_name,
                           code=s_code)
            else:
                return "#  {description} type = {trim_type}\n" \
                       "data[\"{trim_name}\"] = {item_name}{name}\n".\
                    format(description=self.description,
                           trim_type=self.trim_type,
                           trim_name=self.trim_name,
                           item_name=item_name,
                           name=self.name)


    def get_unpack_dict_code(self, ret_name):
        if self.trim_type == "timestamp":
            return ""
        else:
            if self.full_type == "Qot_Common.Security":
                s_code = "#  {description} type={trim_type}\n" \
                         "{ret_name}[\"{trim_name}\"]= merge_qot_mkt_stock_str({trim_name}.market, {trim_name}.code)\n". \
                    format(description=self.description,
                           trim_type=self.trim_type,
                           trim_name=self.trim_name,
                           ret_name=ret_name)
            else:
                s_code = "#  {description} type={trim_type}\n" \
                         "{ret_name}[\"{trim_name}\"]=rsp_pb.s2c.{name}\n".\
                    format(description=self.description,
                           trim_type=self.trim_type,
                           trim_name=self.trim_name,
                           name=self.name,
                           ret_name=ret_name)
            return code_add_space(s_code, 2)


class GenerateCode(object):

    def __init__(self, class_name, prefix=""):
        print("GenerateCode class_name = {} | prefix = {}".format(class_name, prefix))
        self.local_path = os.path.dirname(os.path.realpath(__file__))
        self.common_pb_path = os.path.abspath(os.path.join(self.local_path, "../common/pb/"))
        self.out_put_path = os.path.join(self.local_path, "code")
        if not os.path.exists(self.out_put_path):
            os.makedirs(self.out_put_path)

        self.enums = list()
        self.class_name = class_name
        self.prefix = prefix
        if prefix != "":
            self.json_filename = "{}_{}.proto.json".format(prefix, class_name)
        else:
            self.json_filename = "{}.proto.json".format(class_name)
        self.pb_prefix_name = "{}_pb2.".format(class_name)
        self.obj = None
        self.c2s = self.s2c = None

    def load(self):
        self.obj = None
        with open(os.path.join(self.common_pb_path, self.json_filename), 'r', encoding='UTF-8') as load_f:
            self.obj = json.load(load_f)
            self.load_enums()
            self.load_parameter()
            for i in range(10):
                self.set_parameter_class(self.obj)  # 嵌套重复元素
        with open(os.path.join(self.common_pb_path, "Qot_Common.proto.json"), 'r', encoding='UTF-8') as load_f:
            self.set_parameter_class(json.load(load_f))
        with open(os.path.join(self.common_pb_path, "Trd_Common.proto.json"), 'r', encoding='UTF-8') as load_f:
            self.set_parameter_class(json.load(load_f))

    def load_enums(self):
        self.enums.clear()
        if self.obj is not None:
            files_json = self.obj["files"]
            enums_json = files_json[0]["enums"]
            for item in enums_json:
                c = ClassItemStruct(item, "enums", self.pb_prefix_name)
                self.enums.append(c)

    def load_parameter(self):
        self.c2s = self.s2c = None
        if self.obj is not None:
            files_json = self.obj["files"]
            messages_json = files_json[0]["messages"]
            for item in messages_json:
                long_name = item["longName"]
                if long_name == "C2S":
                    self.c2s = ClassItemStruct(item, "class", self.pb_prefix_name)
                if long_name == "S2C":
                    self.s2c = ClassItemStruct(item, "class", self.pb_prefix_name)

    def set_parameter_class(self, obj):
        if obj is None or self.s2c is None or self.c2s is None:
            return

        files_json = obj["files"]
        messages_json = files_json[0]["messages"]

        for item in messages_json:
            class_obj = ClassItemStruct(item, "class", self.pb_prefix_name)
            self.s2c.set_parameter_class(class_obj)
            self.c2s.set_parameter_class(class_obj)

    def out_class(self):
        if self.c2s is None or self.s2c is None:
            return

        export_file = os.path.join(self.out_put_path, "{}.py".
                                   format(change_variable_name(self.class_name).lower()))
        code_file = open(export_file, 'w', encoding='utf-8')
        function_name = change_variable_name(self.class_name).lower()  # 函数名
        kargs = parameter = warning_filter = ""
        notes = "        " + self.class_name
        pack_req_filter = pack_req_add = ""

        if self.prefix != "":
            pb_file_name = "{}_{}".format(self.prefix, self.class_name)
        else:
            pb_file_name = "{}".format(self.class_name)

        # 大于5的时候变成结构体
        for i in range(len(self.c2s.values)):
            v = self.c2s.values[i]
            parameter += v.get_parameter_name()
            warning_filter += v.get_warning_filter()
            kargs += v.get_args()

            s = v.get_pack_req_filter()
            pack_req_filter += '        \"\"\"check {1} {0}\"\"\"\n'.format(v.description, v.trim_name)
            pack_req_filter += s

            pack_req_add += v.get_pack_req_add()
            if i < (len(self.c2s.values) - 1):
                parameter += ","
                kargs += ",\n"

        # with open(os.path.join(self.local_path, __TemplateFileFunctionName__), 'r', encoding='UTF-8') as load_f:
        template_code = template["class_function"]
        unpack_code = code_add_space(self.s2c.get_unpack_code().rstrip('\n'),1)
        code_str = template_code.format(function_name=function_name,
                                        parameter=parameter,
                                        notes=notes,
                                        warning_filter=warning_filter,
                                        class_name=self.class_name,
                                        kargs=kargs,
                                        get_function_return=self.s2c.get_function_return(),
                                        pack_req_filter=pack_req_filter,
                                        pack_req_add=pack_req_add,
                                        pb_file_name=pb_file_name,
                                        get_unpack_code= unpack_code,
                                        get_unpack_return=self.s2c.get_unpack_return())

        code_file.write(code_str)
        code_file.close()

    def out_enums(self):
        if len(self.enums) == 0:
            return
        export_file = os.path.join(self.out_put_path, "{}_enums.py".format(change_variable_name(self.class_name).lower()))
        with open(os.path.join(self.local_path, __TemplateCodeFileName__), 'r', encoding='UTF-8') as load_f:
            template_code = load_f.read()
        code_file = open(export_file, 'w', encoding='utf-8')
        """写入文件头"""
        with open(os.path.join(self.local_path, __TemplateFileHeadName__), 'r', encoding='UTF-8') as f:
            code_file.write(f.read())

        for class_item in self.enums:
            class_description = class_item.description.replace("\n", "，")
            class_name = class_item.name
            variable_code = ""
            dic_code = ""
            code_file.write("\n\n\n'''-------------------------{}----------------------------'''\n\n\n"
                            .format(class_name))

            for item in class_item.values:
                variablename = item.underscore_name
                strvalue = item.str
                description = item.description
                pbvalue = item.pb_value
                kvcode = "{variablename} = \"{strvalue}\"".format(variablename=variablename, strvalue=strvalue)
                # < 50表示左对齐50个空格

                code = "    {kvcode: <50} # {description}\n".format(kvcode=kvcode, description=description)
                variable_code += code

                code = "            self.{variablename}: {pbvalue},\n".format(
                    variablename=variablename, pbvalue=pbvalue)
                dic_code += code

            c = template_code.format(classdescription=class_description,
                                 classname=class_name,
                                 variablecode=variable_code.rstrip("\n,"),
                                 diccode=dic_code.rstrip("\n,"))
            code_file.write(c)
        code_file.close()

    def rst_enums(self):
        if len(self.enums) == 0:
            return
        export_file = os.path.join(self.out_put_path, "{}_enums.rst".format(change_variable_name(self.class_name).lower()))
        rst_template= ""
        with open(os.path.join(self.local_path, __TemplateRstName__), 'r', encoding='UTF-8') as load_f:
            rst_template = load_f.read()
        code_file = open(export_file, 'w', encoding='utf-8')
        for class_item in self.enums:
            classdescription = class_item.description.replace("\n", "，")
            classname = class_item.name
            dicinfo = ""
            for item in class_item.values:
                name = item.underscore_name
                description = item.description
                kvcode = " ..  py:attribute:: {name}\n\n  {description}\n\n".format(name=name, description=description)
                dicinfo += kvcode

            c = rst_template.format(classname=classname,
                                    classdescription=classdescription,
                                    dicinfo=dicinfo)
            code_file.write(c)
        code_file.close()

    def rst_class(self):
        if self.c2s is None or self.s2c is None:
            return

        export_file = os.path.join(self.out_put_path, "{}.rst".
                                   format(change_variable_name(self.class_name).lower()))
        code_file = open(export_file, 'w', encoding='utf-8')
        code_str = ""
        for i in range(len(self.s2c.values)):
            v = self.s2c.values[i]
            description = v.description
            python_type = v.python_type
            name = v.trim_name
            line = " {:<32}{:<15}{}\n".format(name, python_type, description)
            code_str += line

        code_file.write(code_str)
        code_file.close()

    def save(self):
        self.out_enums()
        self.out_class()
        self.rst_enums()
        self.rst_class()


class Generate(object):
    def __init__(self):
        self.local_path = os.path.dirname(os.path.realpath(__file__))
        self.common_pb_path = os.path.abspath(os.path.join(self.local_path, "../common/pb/"))
        self.pb_list = list()

        """统统载入进来再说"""
        f_list = os.listdir(self.common_pb_path)
        for f in f_list:
            if os.path.splitext(f)[1] == '.proto':
                self.pb_list.append(f)

    def find_pb_by_names(self, names):
        f_list = list()
        for f in self.pb_list:
            (_, file_name) = os.path.split(f)
            for s in names:
                if f.find(s) != -1:
                    f_list.append(f)
                    break
        return f_list

    def generate_json(self, names=None):
        if names is not None:
            f_list = self.find_pb_by_names(names)
        else:
            f_list = self.pb_list

        for f in f_list:
            (_, file_name) = os.path.split(f)

            cmd = '''cd /d {work_path} & \
                  protoc.exe -I=. --python_out=./  {file_name} & \
                  protoc.exe --doc_out=. --doc_opt=json,{file_name}.json ./{file_name} &'''\
                .format(file_name=file_name,
                        work_path=self.common_pb_path)
            os.system(cmd)
            print(cmd)

    def generate_code(self, names):
        if names is not None:
            pb_list = self.find_pb_by_names(names)
        else:
            pb_list = self.pb_list

        for f in pb_list:
            (_, file_name) = os.path.split(f)
            json_file_path = os.path.join(self.common_pb_path, file_name + ".json")
            if not os.path.exists(json_file_path):
                print(json_file_path + " not exists")
                continue
            (name, _) = os.path.splitext(file_name)

            if name.find("Qot_") == 0:
                prefix = "Qot"
                class_name = name[4:]
            elif name.find("Trd_") == 0:
                prefix = "Trd"
                class_name = name[4:]
            else:
                prefix = ""
                class_name = name

            if class_name in ["Common", "Sub"]:
                continue
            code = GenerateCode(class_name, prefix)
            code.load()
            code.save()


def generate(names):
    if isinstance(names, str):
        names = [names]
    gener = Generate()
    gener.generate_json(names)
    gener.generate_code(names)


if __name__ =="__main__":
    generate(None)
    # c = GenerateCode("GetOwnerPlate", "Qot")
    # c.load()
    # c.save()









# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Qot_GetFutureInfo.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Common_pb2 as Common__pb2
import Qot_Common_pb2 as Qot__Common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Qot_GetFutureInfo.proto',
  package='Qot_GetFutureInfo',
  syntax='proto2',
  serialized_pb=_b('\n\x17Qot_GetFutureInfo.proto\x12\x11Qot_GetFutureInfo\x1a\x0c\x43ommon.proto\x1a\x10Qot_Common.proto\"\'\n\tTradeTime\x12\r\n\x05\x62\x65gin\x18\x01 \x01(\x01\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x01\"\xb2\x03\n\nFutureInfo\x12\x0c\n\x04name\x18\x01 \x02(\t\x12&\n\x08security\x18\x02 \x02(\x0b\x32\x14.Qot_Common.Security\x12\x15\n\rlastTradeTime\x18\x03 \x02(\t\x12\x1a\n\x12lastTradeTimestamp\x18\x04 \x01(\x01\x12#\n\x05owner\x18\x05 \x01(\x0b\x32\x14.Qot_Common.Security\x12\x12\n\nownerOther\x18\x06 \x02(\t\x12\x10\n\x08\x65xchange\x18\x07 \x02(\t\x12\x14\n\x0c\x63ontractType\x18\x08 \x02(\t\x12\x14\n\x0c\x63ontractSize\x18\t \x02(\x01\x12\x18\n\x10\x63ontractSizeUnit\x18\n \x02(\t\x12\x15\n\rquoteCurrency\x18\x0b \x02(\t\x12\x0e\n\x06minVar\x18\x0c \x02(\x01\x12\x12\n\nminVarUnit\x18\r \x02(\t\x12\x11\n\tquoteUnit\x18\x0e \x01(\t\x12/\n\ttradeTime\x18\x0f \x03(\x0b\x32\x1c.Qot_GetFutureInfo.TradeTime\x12\x10\n\x08timeZone\x18\x10 \x02(\t\x12\x19\n\x11\x65xchangeFormatUrl\x18\x11 \x02(\t\"1\n\x03\x43\x32S\x12*\n\x0csecurityList\x18\x01 \x03(\x0b\x32\x14.Qot_Common.Security\"<\n\x03S2C\x12\x35\n\x0e\x66utureInfoList\x18\x01 \x03(\x0b\x32\x1d.Qot_GetFutureInfo.FutureInfo\".\n\x07Request\x12#\n\x03\x63\x32s\x18\x01 \x02(\x0b\x32\x16.Qot_GetFutureInfo.C2S\"g\n\x08Response\x12\x15\n\x07retType\x18\x01 \x02(\x05:\x04-400\x12\x0e\n\x06retMsg\x18\x02 \x01(\t\x12\x0f\n\x07\x65rrCode\x18\x03 \x01(\x05\x12#\n\x03s2c\x18\x04 \x01(\x0b\x32\x16.Qot_GetFutureInfo.S2CBG\n\x13\x63om.futu.openapi.pbZ0github.com/futuopen/ftapi4go/pb/qotgetfutureinfo')
  ,
  dependencies=[Common__pb2.DESCRIPTOR,Qot__Common__pb2.DESCRIPTOR,])




_TRADETIME = _descriptor.Descriptor(
  name='TradeTime',
  full_name='Qot_GetFutureInfo.TradeTime',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='begin', full_name='Qot_GetFutureInfo.TradeTime.begin', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end', full_name='Qot_GetFutureInfo.TradeTime.end', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=78,
  serialized_end=117,
)


_FUTUREINFO = _descriptor.Descriptor(
  name='FutureInfo',
  full_name='Qot_GetFutureInfo.FutureInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Qot_GetFutureInfo.FutureInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='security', full_name='Qot_GetFutureInfo.FutureInfo.security', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lastTradeTime', full_name='Qot_GetFutureInfo.FutureInfo.lastTradeTime', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lastTradeTimestamp', full_name='Qot_GetFutureInfo.FutureInfo.lastTradeTimestamp', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='owner', full_name='Qot_GetFutureInfo.FutureInfo.owner', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ownerOther', full_name='Qot_GetFutureInfo.FutureInfo.ownerOther', index=5,
      number=6, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='exchange', full_name='Qot_GetFutureInfo.FutureInfo.exchange', index=6,
      number=7, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contractType', full_name='Qot_GetFutureInfo.FutureInfo.contractType', index=7,
      number=8, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contractSize', full_name='Qot_GetFutureInfo.FutureInfo.contractSize', index=8,
      number=9, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contractSizeUnit', full_name='Qot_GetFutureInfo.FutureInfo.contractSizeUnit', index=9,
      number=10, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='quoteCurrency', full_name='Qot_GetFutureInfo.FutureInfo.quoteCurrency', index=10,
      number=11, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='minVar', full_name='Qot_GetFutureInfo.FutureInfo.minVar', index=11,
      number=12, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='minVarUnit', full_name='Qot_GetFutureInfo.FutureInfo.minVarUnit', index=12,
      number=13, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='quoteUnit', full_name='Qot_GetFutureInfo.FutureInfo.quoteUnit', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tradeTime', full_name='Qot_GetFutureInfo.FutureInfo.tradeTime', index=14,
      number=15, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timeZone', full_name='Qot_GetFutureInfo.FutureInfo.timeZone', index=15,
      number=16, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='exchangeFormatUrl', full_name='Qot_GetFutureInfo.FutureInfo.exchangeFormatUrl', index=16,
      number=17, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=120,
  serialized_end=554,
)


_C2S = _descriptor.Descriptor(
  name='C2S',
  full_name='Qot_GetFutureInfo.C2S',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='securityList', full_name='Qot_GetFutureInfo.C2S.securityList', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=556,
  serialized_end=605,
)


_S2C = _descriptor.Descriptor(
  name='S2C',
  full_name='Qot_GetFutureInfo.S2C',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='futureInfoList', full_name='Qot_GetFutureInfo.S2C.futureInfoList', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=607,
  serialized_end=667,
)


_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='Qot_GetFutureInfo.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='c2s', full_name='Qot_GetFutureInfo.Request.c2s', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=669,
  serialized_end=715,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='Qot_GetFutureInfo.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='retType', full_name='Qot_GetFutureInfo.Response.retType', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-400,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='retMsg', full_name='Qot_GetFutureInfo.Response.retMsg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errCode', full_name='Qot_GetFutureInfo.Response.errCode', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='s2c', full_name='Qot_GetFutureInfo.Response.s2c', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=717,
  serialized_end=820,
)

_FUTUREINFO.fields_by_name['security'].message_type = Qot__Common__pb2._SECURITY
_FUTUREINFO.fields_by_name['owner'].message_type = Qot__Common__pb2._SECURITY
_FUTUREINFO.fields_by_name['tradeTime'].message_type = _TRADETIME
_C2S.fields_by_name['securityList'].message_type = Qot__Common__pb2._SECURITY
_S2C.fields_by_name['futureInfoList'].message_type = _FUTUREINFO
_REQUEST.fields_by_name['c2s'].message_type = _C2S
_RESPONSE.fields_by_name['s2c'].message_type = _S2C
DESCRIPTOR.message_types_by_name['TradeTime'] = _TRADETIME
DESCRIPTOR.message_types_by_name['FutureInfo'] = _FUTUREINFO
DESCRIPTOR.message_types_by_name['C2S'] = _C2S
DESCRIPTOR.message_types_by_name['S2C'] = _S2C
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TradeTime = _reflection.GeneratedProtocolMessageType('TradeTime', (_message.Message,), dict(
  DESCRIPTOR = _TRADETIME,
  __module__ = 'Qot_GetFutureInfo_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetFutureInfo.TradeTime)
  ))
_sym_db.RegisterMessage(TradeTime)

FutureInfo = _reflection.GeneratedProtocolMessageType('FutureInfo', (_message.Message,), dict(
  DESCRIPTOR = _FUTUREINFO,
  __module__ = 'Qot_GetFutureInfo_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetFutureInfo.FutureInfo)
  ))
_sym_db.RegisterMessage(FutureInfo)

C2S = _reflection.GeneratedProtocolMessageType('C2S', (_message.Message,), dict(
  DESCRIPTOR = _C2S,
  __module__ = 'Qot_GetFutureInfo_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetFutureInfo.C2S)
  ))
_sym_db.RegisterMessage(C2S)

S2C = _reflection.GeneratedProtocolMessageType('S2C', (_message.Message,), dict(
  DESCRIPTOR = _S2C,
  __module__ = 'Qot_GetFutureInfo_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetFutureInfo.S2C)
  ))
_sym_db.RegisterMessage(S2C)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'Qot_GetFutureInfo_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetFutureInfo.Request)
  ))
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'Qot_GetFutureInfo_pb2'
  # @@protoc_insertion_point(class_scope:Qot_GetFutureInfo.Response)
  ))
_sym_db.RegisterMessage(Response)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\023com.futu.openapi.pbZ0github.com/futuopen/ftapi4go/pb/qotgetfutureinfo'))
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
"""
这个文件主要在开发futu-api库的时候用得上，如果你只是使用futu-api库，可以不用引入这个文件
"""
import os
import sys


def get_package_path(name):
    temp_path = os.getcwd()
    for i in range(0, 3):  # 最多查找3级
        temp_path = os.path.abspath(os.path.dirname(temp_path))
        filename = os.path.basename(temp_path)
        if filename == name:
            break
    return temp_path


#我們不再提供回测框架，这里的回测mini库仅用于demo演示
def add_tiny_quant_frame():
    temp_path = get_package_path("tiny_quant")
    os.path.join(temp_path, "tiny_quant_frame")
    if (temp_path.strip() != '') and (temp_path not in sys.path):
        print(temp_path)
        sys.path.append(temp_path)


#这一段代码是为了开发库的过程中，优先匹配当前工作路径的futu-api库
def add_futu_package_path():
    temp_path = os.path.abspath(os.path.dirname(get_package_path("futu")))
    if (temp_path.strip() != '') and (temp_path not in sys.path):
        sys.path.insert(0, temp_path)
        print(temp_path)




# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Common.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='Common.proto',
  package='Common',
  syntax='proto2',
  serialized_pb=_b('\n\x0c\x43ommon.proto\x12\x06\x43ommon\",\n\x08PacketID\x12\x0e\n\x06\x63onnID\x18\x01 \x02(\x04\x12\x10\n\x08serialNo\x18\x02 \x02(\r\"L\n\rProgramStatus\x12\'\n\x04type\x18\x01 \x02(\x0e\x32\x19.Common.ProgramStatusType\x12\x12\n\nstrExtDesc\x18\x02 \x01(\t*\xb6\x01\n\x07RetType\x12\x13\n\x0fRetType_Succeed\x10\x00\x12\x1b\n\x0eRetType_Failed\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x1c\n\x0fRetType_TimeOut\x10\x9c\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x1f\n\x12RetType_DisConnect\x10\xb8\xfe\xff\xff\xff\xff\xff\xff\xff\x01\x12\x1c\n\x0fRetType_Unknown\x10\xf0\xfc\xff\xff\xff\xff\xff\xff\xff\x01\x12\x1c\n\x0fRetType_Invalid\x10\x8c\xfc\xff\xff\xff\xff\xff\xff\xff\x01*\x83\x01\n\rPacketEncAlgo\x12\x1b\n\x17PacketEncAlgo_FTAES_ECB\x10\x00\x12\x1f\n\x12PacketEncAlgo_None\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x19\n\x15PacketEncAlgo_AES_ECB\x10\x01\x12\x19\n\x15PacketEncAlgo_AES_CBC\x10\x02*4\n\x08ProtoFmt\x12\x15\n\x11ProtoFmt_Protobuf\x10\x00\x12\x11\n\rProtoFmt_Json\x10\x01*\xf0\x03\n\x11ProgramStatusType\x12\x1a\n\x16ProgramStatusType_None\x10\x00\x12\x1c\n\x18ProgramStatusType_Loaded\x10\x01\x12\x1c\n\x18ProgramStatusType_Loging\x10\x02\x12\'\n#ProgramStatusType_NeedPicVerifyCode\x10\x03\x12)\n%ProgramStatusType_NeedPhoneVerifyCode\x10\x04\x12!\n\x1dProgramStatusType_LoginFailed\x10\x05\x12!\n\x1dProgramStatusType_ForceUpdate\x10\x06\x12*\n&ProgramStatusType_NessaryDataPreparing\x10\x07\x12(\n$ProgramStatusType_NessaryDataMissing\x10\x08\x12\'\n#ProgramStatusType_UnAgreeDisclaimer\x10\t\x12\x1b\n\x17ProgramStatusType_Ready\x10\n\x12!\n\x1dProgramStatusType_ForceLogout\x10\x0b\x12*\n&ProgramStatusType_DisclaimerPullFailed\x10\x0c\x42=\n\x13\x63om.futu.openapi.pbZ&github.com/futuopen/ftapi4go/pb/common')
)

_RETTYPE = _descriptor.EnumDescriptor(
  name='RetType',
  full_name='Common.RetType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RetType_Succeed', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RetType_Failed', index=1, number=-1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RetType_TimeOut', index=2, number=-100,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RetType_DisConnect', index=3, number=-200,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RetType_Unknown', index=4, number=-400,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RetType_Invalid', index=5, number=-500,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=149,
  serialized_end=331,
)
_sym_db.RegisterEnumDescriptor(_RETTYPE)

RetType = enum_type_wrapper.EnumTypeWrapper(_RETTYPE)
_PACKETENCALGO = _descriptor.EnumDescriptor(
  name='PacketEncAlgo',
  full_name='Common.PacketEncAlgo',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PacketEncAlgo_FTAES_ECB', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PacketEncAlgo_None', index=1, number=-1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PacketEncAlgo_AES_ECB', index=2, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PacketEncAlgo_AES_CBC', index=3, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=334,
  serialized_end=465,
)
_sym_db.RegisterEnumDescriptor(_PACKETENCALGO)

PacketEncAlgo = enum_type_wrapper.EnumTypeWrapper(_PACKETENCALGO)
_PROTOFMT = _descriptor.EnumDescriptor(
  name='ProtoFmt',
  full_name='Common.ProtoFmt',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ProtoFmt_Protobuf', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProtoFmt_Json', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=467,
  serialized_end=519,
)
_sym_db.RegisterEnumDescriptor(_PROTOFMT)

ProtoFmt = enum_type_wrapper.EnumTypeWrapper(_PROTOFMT)
_PROGRAMSTATUSTYPE = _descriptor.EnumDescriptor(
  name='ProgramStatusType',
  full_name='Common.ProgramStatusType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_None', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_Loaded', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_Loging', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_NeedPicVerifyCode', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_NeedPhoneVerifyCode', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_LoginFailed', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_ForceUpdate', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_NessaryDataPreparing', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_NessaryDataMissing', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_UnAgreeDisclaimer', index=9, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_Ready', index=10, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_ForceLogout', index=11, number=11,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ProgramStatusType_DisclaimerPullFailed', index=12, number=12,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=522,
  serialized_end=1018,
)
_sym_db.RegisterEnumDescriptor(_PROGRAMSTATUSTYPE)

ProgramStatusType = enum_type_wrapper.EnumTypeWrapper(_PROGRAMSTATUSTYPE)
RetType_Succeed = 0
RetType_Failed = -1
RetType_TimeOut = -100
RetType_DisConnect = -200
RetType_Unknown = -400
RetType_Invalid = -500
PacketEncAlgo_FTAES_ECB = 0
PacketEncAlgo_None = -1
PacketEncAlgo_AES_ECB = 1
PacketEncAlgo_AES_CBC = 2
ProtoFmt_Protobuf = 0
ProtoFmt_Json = 1
ProgramStatusType_None = 0
ProgramStatusType_Loaded = 1
ProgramStatusType_Loging = 2
ProgramStatusType_NeedPicVerifyCode = 3
ProgramStatusType_NeedPhoneVerifyCode = 4
ProgramStatusType_LoginFailed = 5
ProgramStatusType_ForceUpdate = 6
ProgramStatusType_NessaryDataPreparing = 7
ProgramStatusType_NessaryDataMissing = 8
ProgramStatusType_UnAgreeDisclaimer = 9
ProgramStatusType_Ready = 10
ProgramStatusType_ForceLogout = 11
ProgramStatusType_DisclaimerPullFailed = 12



_PACKETID = _descriptor.Descriptor(
  name='PacketID',
  full_name='Common.PacketID',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='connID', full_name='Common.PacketID.connID', index=0,
      number=1, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='serialNo', full_name='Common.PacketID.serialNo', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24,
  serialized_end=68,
)


_PROGRAMSTATUS = _descriptor.Descriptor(
  name='ProgramStatus',
  full_name='Common.ProgramStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='Common.ProgramStatus.type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='strExtDesc', full_name='Common.ProgramStatus.strExtDesc', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=70,
  serialized_end=146,
)

_PROGRAMSTATUS.fields_by_name['type'].enum_type = _PROGRAMSTATUSTYPE
DESCRIPTOR.message_types_by_name['PacketID'] = _PACKETID
DESCRIPTOR.message_types_by_name['ProgramStatus'] = _PROGRAMSTATUS
DESCRIPTOR.enum_types_by_name['RetType'] = _RETTYPE
DESCRIPTOR.enum_types_by_name['PacketEncAlgo'] = _PACKETENCALGO
DESCRIPTOR.enum_types_by_name['ProtoFmt'] = _PROTOFMT
DESCRIPTOR.enum_types_by_name['ProgramStatusType'] = _PROGRAMSTATUSTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PacketID = _reflection.GeneratedProtocolMessageType('PacketID', (_message.Message,), dict(
  DESCRIPTOR = _PACKETID,
  __module__ = 'Common_pb2'
  # @@protoc_insertion_point(class_scope:Common.PacketID)
  ))
_sym_db.RegisterMessage(PacketID)

ProgramStatus = _reflection.GeneratedProtocolMessageType('ProgramStatus', (_message.Message,), dict(
  DESCRIPTOR = _PROGRAMSTATUS,
  __module__ = 'Common_pb2'
  # @@protoc_insertion_point(class_scope:Common.ProgramStatus)
  ))
_sym_db.RegisterMessage(ProgramStatus)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\023com.futu.openapi.pbZ&github.com/futuopen/ftapi4go/pb/common'))
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
from futu.quote.quote_response_handler import *
from futu.trade.trade_response_handler import *


class HandlerContext:
    """Handle Context"""

    def __init__(self, cb_check_recv):
        self.cb_check_recv = cb_check_recv
        self._default_handler = RspHandlerBase()
        self._handler_table = {
            1003: {
                "type": SysNotifyHandlerBase,
                "obj": SysNotifyHandlerBase()
            },
            1004: {
                "type": KeepAliveHandlerBase,
                "obj": KeepAliveHandlerBase()
            },
            2208: {
                "type": TradeOrderHandlerBase,
                "obj": TradeOrderHandlerBase()
            },
            2218: {
                "type": TradeDealHandlerBase,
                "obj": TradeDealHandlerBase()
            },

            3005: {
                "type": StockQuoteHandlerBase,
                "obj": StockQuoteHandlerBase()
            },
            3007: {
                "type": CurKlineHandlerBase,
                "obj": CurKlineHandlerBase()
            },
            3009: {
                "type": RTDataHandlerBase,
                "obj": RTDataHandlerBase()
            },
            3011: {
                "type": TickerHandlerBase,
                "obj": TickerHandlerBase()
            },
            3013: {
                "type": OrderBookHandlerBase,
                "obj": OrderBookHandlerBase()
            },
            3015: {
                "type": BrokerHandlerBase,
                "obj": BrokerHandlerBase()
            },
            3019: {
                "type": PriceReminderHandlerBase,
                "obj": PriceReminderHandlerBase()
            }
            # 3017: {
            #     "type": OrderDetailHandlerBase,
            #     "obj": OrderDetailHandlerBase()
            # }
        }

    def set_handler(self, handler):
        """
        set the callback processing object to be used by the receiving thread after receiving the data.User should set
        their own callback object setting in order to achieve event driven.
        :param handler:the object in callback handler base
        :return: ret_error or ret_ok
        """
        set_flag = False
        for protoc in self._handler_table:
            if isinstance(handler, self._handler_table[protoc]["type"]):
                self._handler_table[protoc]["obj"] = handler
                return RET_OK

        if set_flag is False:
            return RET_ERROR

    def recv_func(self, rsp_pb, proto_id):
        """receive response callback function"""

        if self.cb_check_recv is not None and not self.cb_check_recv() and ProtoId.is_proto_id_push(proto_id):
            return

        handler = self._default_handler
        if proto_id in self._handler_table:
            handler = self._handler_table[proto_id]['obj']
        handler.on_recv_rsp(rsp_pb)

    @staticmethod
    def error_func(err_str):
        """error callback function"""
        print(err_str)




# -*- coding: utf-8 -*-

import sys
import io
import time
import json
from .ft_logger import *

IS_PY2 = sys.version_info[0] == 2
debug_model = False

def bytes_utf8(data):
    if IS_PY2:
        return bytes(data)
    else:
        return bytes(data, encoding='utf-8')


def str_utf8(data):
    if IS_PY2:
        return str(data)
    else:
        return str(data, encoding='utf-8')

def set_debug_model(on_off):
    global debug_model
    debug_model = (on_off is True)
    logger.debug_model = (on_off is True)


class RspHandlerBase(object):
    """callback function base class"""

    def __init__(self):
        self._file = None
        self._current_day = 0

    def __del__(self):
        self._close_file()

    def _close_file(self):
        if self._file is not None:
            self._file.close()
            self._file = None

    def _open_file(self):
        self._close_file()

        time_str = time.strftime('%Y_%m_%d', time.localtime(time.time()))
        millis = int(round(time.time() * 1000))
        type_name = self.__class__.__name__
        log_file_name = os.path.join("Track_{}_{}_{}.log".format(time_str, millis, type_name))
        file_path = os.path.join(logger.log_path, log_file_name)
        self._file = open(file_path, 'w')
        self._current_day = time.localtime().tm_mday

    def on_recv_log(self, content):
    #用于记录推送信息，性能不高，如果需要提升性能的时候，记得把set_futu_debug_model关掉
        if not debug_model:
            return
        current_day = time.localtime().tm_mday

        if (self._file is None) or (current_day != self._current_day):
            self._open_file()

        if self._file:
            try:
                log_contents = dict()
                log_contents["content"] = content
                log_contents["time"] = time.strftime('%H:%M:%S', time.localtime(time.time()))
                log_contents["millis"] = int(round(time.time() * 1000))
                self._file.write(json.dumps(log_contents, indent=4).replace('\n', '').replace('\t', '').replace(' ', '') + "\n")
                self._file.flush()
            except TypeError:
                type_name = self.__class__.__name__
                self._file.write("{} can not dump json\n".format(type_name))
                self._file.flush()
            except IOError:
                self.close_file()
            except Exception as e:
                logger.error(str(e).replace('\n', '').replace('\t', ''))


    def on_recv_rsp(self, rsp_pb):
        """receive response callback function"""
        return 0, None





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: GetUserInfo.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Common_pb2 as Common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='GetUserInfo.proto',
  package='GetUserInfo',
  syntax='proto2',
  serialized_pb=_b('\n\x11GetUserInfo.proto\x12\x0bGetUserInfo\x1a\x0c\x43ommon.proto\"\x13\n\x03\x43\x32S\x12\x0c\n\x04\x66lag\x18\x02 \x01(\x05\"\x92\x03\n\x03S2C\x12\x10\n\x08nickName\x18\x01 \x01(\t\x12\x11\n\tavatarUrl\x18\x02 \x01(\t\x12\x10\n\x08\x61piLevel\x18\x03 \x01(\t\x12\x12\n\nhkQotRight\x18\x04 \x01(\x05\x12\x12\n\nusQotRight\x18\x05 \x01(\x05\x12\x12\n\ncnQotRight\x18\x06 \x01(\x05\x12\x1d\n\x15isNeedAgreeDisclaimer\x18\x07 \x01(\x08\x12\x0e\n\x06userID\x18\x08 \x01(\x03\x12\x12\n\nupdateType\x18\t \x01(\x05\x12\x0e\n\x06webKey\x18\n \x01(\t\x12\x16\n\x0ewebJumpUrlHead\x18\x12 \x01(\t\x12\x18\n\x10hkOptionQotRight\x18\x0b \x01(\x05\x12\x1b\n\x13hasUSOptionQotRight\x18\x0c \x01(\x08\x12\x18\n\x10hkFutureQotRight\x18\r \x01(\x05\x12\x10\n\x08subQuota\x18\x0e \x01(\x05\x12\x16\n\x0ehistoryKLQuota\x18\x0f \x01(\x05\x12\x18\n\x10usFutureQotRight\x18\x10 \x01(\x05\x12\x18\n\x10usOptionQotRight\x18\x11 \x01(\x05\"(\n\x07Request\x12\x1d\n\x03\x63\x32s\x18\x01 \x02(\x0b\x32\x10.GetUserInfo.C2S\"a\n\x08Response\x12\x15\n\x07retType\x18\x01 \x02(\x05:\x04-400\x12\x0e\n\x06retMsg\x18\x02 \x01(\t\x12\x0f\n\x07\x65rrCode\x18\x03 \x01(\x05\x12\x1d\n\x03s2c\x18\x04 \x01(\x0b\x32\x10.GetUserInfo.S2C*N\n\nUpdateType\x12\x13\n\x0fUpdateType_None\x10\x00\x12\x15\n\x11UpdateType_Advice\x10\x01\x12\x14\n\x10UpdateType_Force\x10\x02*\xae\x01\n\rUserInfoField\x12\x17\n\x13UserInfoField_Basic\x10\x01\x12\x15\n\x11UserInfoField_API\x10\x02\x12\x1a\n\x16UserInfoField_QotRight\x10\x04\x12\x1c\n\x18UserInfoField_Disclaimer\x10\x08\x12\x18\n\x14UserInfoField_Update\x10\x10\x12\x19\n\x14UserInfoField_WebKey\x10\x80\x10\x42\x42\n\x13\x63om.futu.openapi.pbZ+github.com/futuopen/ftapi4go/pb/getuserinfo')
  ,
  dependencies=[Common__pb2.DESCRIPTOR,])

_UPDATETYPE = _descriptor.EnumDescriptor(
  name='UpdateType',
  full_name='GetUserInfo.UpdateType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UpdateType_None', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UpdateType_Advice', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UpdateType_Force', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=615,
  serialized_end=693,
)
_sym_db.RegisterEnumDescriptor(_UPDATETYPE)

UpdateType = enum_type_wrapper.EnumTypeWrapper(_UPDATETYPE)
_USERINFOFIELD = _descriptor.EnumDescriptor(
  name='UserInfoField',
  full_name='GetUserInfo.UserInfoField',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UserInfoField_Basic', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UserInfoField_API', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UserInfoField_QotRight', index=2, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UserInfoField_Disclaimer', index=3, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UserInfoField_Update', index=4, number=16,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UserInfoField_WebKey', index=5, number=2048,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=696,
  serialized_end=870,
)
_sym_db.RegisterEnumDescriptor(_USERINFOFIELD)

UserInfoField = enum_type_wrapper.EnumTypeWrapper(_USERINFOFIELD)
UpdateType_None = 0
UpdateType_Advice = 1
UpdateType_Force = 2
UserInfoField_Basic = 1
UserInfoField_API = 2
UserInfoField_QotRight = 4
UserInfoField_Disclaimer = 8
UserInfoField_Update = 16
UserInfoField_WebKey = 2048



_C2S = _descriptor.Descriptor(
  name='C2S',
  full_name='GetUserInfo.C2S',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='flag', full_name='GetUserInfo.C2S.flag', index=0,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=48,
  serialized_end=67,
)


_S2C = _descriptor.Descriptor(
  name='S2C',
  full_name='GetUserInfo.S2C',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nickName', full_name='GetUserInfo.S2C.nickName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='avatarUrl', full_name='GetUserInfo.S2C.avatarUrl', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='apiLevel', full_name='GetUserInfo.S2C.apiLevel', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hkQotRight', full_name='GetUserInfo.S2C.hkQotRight', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='usQotRight', full_name='GetUserInfo.S2C.usQotRight', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cnQotRight', full_name='GetUserInfo.S2C.cnQotRight', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='isNeedAgreeDisclaimer', full_name='GetUserInfo.S2C.isNeedAgreeDisclaimer', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='userID', full_name='GetUserInfo.S2C.userID', index=7,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='updateType', full_name='GetUserInfo.S2C.updateType', index=8,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='webKey', full_name='GetUserInfo.S2C.webKey', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='webJumpUrlHead', full_name='GetUserInfo.S2C.webJumpUrlHead', index=10,
      number=18, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hkOptionQotRight', full_name='GetUserInfo.S2C.hkOptionQotRight', index=11,
      number=11, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hasUSOptionQotRight', full_name='GetUserInfo.S2C.hasUSOptionQotRight', index=12,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hkFutureQotRight', full_name='GetUserInfo.S2C.hkFutureQotRight', index=13,
      number=13, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='subQuota', full_name='GetUserInfo.S2C.subQuota', index=14,
      number=14, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='historyKLQuota', full_name='GetUserInfo.S2C.historyKLQuota', index=15,
      number=15, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='usFutureQotRight', full_name='GetUserInfo.S2C.usFutureQotRight', index=16,
      number=16, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='usOptionQotRight', full_name='GetUserInfo.S2C.usOptionQotRight', index=17,
      number=17, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=70,
  serialized_end=472,
)


_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='GetUserInfo.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='c2s', full_name='GetUserInfo.Request.c2s', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=474,
  serialized_end=514,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='GetUserInfo.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='retType', full_name='GetUserInfo.Response.retType', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-400,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='retMsg', full_name='GetUserInfo.Response.retMsg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='errCode', full_name='GetUserInfo.Response.errCode', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='s2c', full_name='GetUserInfo.Response.s2c', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=516,
  serialized_end=613,
)

_REQUEST.fields_by_name['c2s'].message_type = _C2S
_RESPONSE.fields_by_name['s2c'].message_type = _S2C
DESCRIPTOR.message_types_by_name['C2S'] = _C2S
DESCRIPTOR.message_types_by_name['S2C'] = _S2C
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.enum_types_by_name['UpdateType'] = _UPDATETYPE
DESCRIPTOR.enum_types_by_name['UserInfoField'] = _USERINFOFIELD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

C2S = _reflection.GeneratedProtocolMessageType('C2S', (_message.Message,), dict(
  DESCRIPTOR = _C2S,
  __module__ = 'GetUserInfo_pb2'
  # @@protoc_insertion_point(class_scope:GetUserInfo.C2S)
  ))
_sym_db.RegisterMessage(C2S)

S2C = _reflection.GeneratedProtocolMessageType('S2C', (_message.Message,), dict(
  DESCRIPTOR = _S2C,
  __module__ = 'GetUserInfo_pb2'
  # @@protoc_insertion_point(class_scope:GetUserInfo.S2C)
  ))
_sym_db.RegisterMessage(S2C)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'GetUserInfo_pb2'
  # @@protoc_insertion_point(class_scope:GetUserInfo.Request)
  ))
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'GetUserInfo_pb2'
  # @@protoc_insertion_point(class_scope:GetUserInfo.Response)
  ))
_sym_db.RegisterMessage(Response)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\023com.futu.openapi.pbZ+github.com/futuopen/ftapi4go/pb/getuserinfo'))
# @@protoc_insertion_point(module_scope)
